#include "charybdis_BHgenerator.h"


/*******************************CONSTRUCTOR***************************************************/
BHgenerator::BHgenerator(ccinterface& console,CRandomMersenne& random):rand(random),LH_head(console.LH_head),pdfholder(console.LH_head),lhevent(console.lhevent),eventhistory(console.eventhistory)
{
  initclass(console);     //initializes all the input variables with the values in the console object
  define_constants();     //computes some initial constants to be used during the run
  
  printruninfo();         //prints some information on the terminal
  LH_head.print_init_tag(lhevent);		
  LH_head.print_init_tag(eventhistory);		//
  initBHstatic();
}

void BHgenerator::initclass(ccinterface &console){
  /********initializes all input parameters in the class********/
  minBHmass=console.minBHmass;                	//minimum BH mass accepted during production (in Gev)
  maxBHmass=console.maxBHmass;                	//maximum BH mass c during production (in Gev)
  NumMaxEvents=console.NumMaxEvents;          	//number of events to generate
  mc_samplesize=console.mc_samplesize;			//samplesize for the monte carlo
  BH_in_LHE=console.BH_in_LHE;                	//defines if the black hole is appears as an intermediary particle on the Les Houches file or not
  verbositycode=console.verbositycode;        	//defines the amount of information to output to cout
  TotalDim=console.TotalDim;                  	//number of dimensions to use
  PlanckMass=console.PlanckMass;              	//planck mass to use (in Gev)
  planckmassdef=console.planckmassdef;        	//switch that defines how planck's mass is calculated
  enhanceXsec=console.enhanceXsec;          	//switch that defines the use Yoshino-Rychkov impact cross-section calculation
  MJlostGraviton=console.MJlostGraviton;      	//switch that defines if there is mass loss threw gravity waves in the balding phase
  useminBHmass=console.useminBHmass;          	//switch that defines if there is a min BH mass at evaporation or if it goes down to planck scale
  cvbias=console.cvbias;                      	//switch that defines the use of the constant angular velocity bias
  isotroploss=console.isotroploss;            	//loss of isotropy threw gravitational radiation
  GTscale=console.GTscale;                    	//switch that defines the use of Giddings-Thomas scale for the PDF's
  nleptonicsv[0]=console.nleptonicsv[0];      	//switchs that define if TOTAL leptonic number[0],
  nleptonicsv[1]=console.nleptonicsv[1];      	//leptonic [1],
  nleptonicsv[2]=console.nleptonicsv[2];      	//muonic  [2] and
  nleptonicsv[3]=console.nleptonicsv[3];      	//tau [3] conservation is imposed
  BHspin=console.BHspin;                      	//switch that defines if the black hole has angular momentum
  BHspinvar=console.BHspinvar;                	//switch that defines if the angular momentum may vary during the evaporation phase
  BHanis=console.BHanis;                      	//switch that defines the use non-uniform angular functions during evaporation
  greybody=console.greybody;                  	//switch that defines the use of grey-body factors
  THvar=console.THvar;                        	//switch that defines if the Hawking's temperature may vary during evaporation
  alloweddecay=console.alloweddecay;          	//switch that defines which particles may be produced during evaporation
  BHrecoil=console.BHrecoil;                  	//switch that defines the BH recoil model
  THmax=console.THmax;                        	//maximum Hawking's temperature allowed
  nbodyaverage=console.nbodyaverage;          	//switch that defines if the average expected particle number should be used to terminate evaporation
  kincut=console.kincut;                      	//switch that defines if the kinematic cut-off should be used to terminate evaporation
  skip2remnant=console.skip2remnant;          	//switch used to skip the evaporation phase directly to the remnant
  RMdecaynum=console.RMdecaynum;               	//defines the number of particles the remnant decays to
  RMphasedecay=console.RMphasedecay;          	//switch that defines if remnant decay uniform ("phase-wise") or depends on remnant spin
  RMvarMultiplicity=console.RMvarMultiplicity;	//switch that defines the use of the variable-multiplicity remnant model or not
  RMboil=console.RMboil;                      	//switch that defines the use of the boiling remnant model or not
  RMstable=console.RMstable;                  	//switch that defines the use of the stable remnant model or not
  RMminmass=console.RMminmass;                	//remnant minimum mass for the boiling model
}

void BHgenerator::printruninfo(){
  //prints some information about the run to the terminal
  //(for a full list of parameters check the output files headers)
  cout<<"Charybdis2cpp Black Hole Event Generator"<<endl
      <<endl
      <<"Run Information:"<<endl
      <<"verbosity  code: "<<verbositycode<<endl
      <<"Beam Energies (Gev): "<<LH_head.Ebmup[0]<<"; "<<LH_head.Ebmup[1]<<endl
      <<"Particle ID's: "<<LH_head.IDbmup[0]<<"; "<<LH_head.IDbmup[1]<<endl
      <<"Black hole mass (Gev) between: "<<minBHmass<<"; "<<maxBHmass<<endl
      <<"Total number of dimensions: "<<TotalDim<<endl
      <<"Planck Mass: "<<PlanckMass<<endl;
  if (enhanceXsec)
    cout<<"Using Yoshino-Rychkov cross-section enhanced calculation."<<endl;
  if (MJlostGraviton)
    cout<<"Assuming loss of mass and momentum through gravitational waves."<<endl;
  cout<<endl;
}

void BHgenerator::initBHstatic(){
/****************initiates the static data for the BHparticle class********************/
	BHparticle::rand=rand;
	BHparticle::enhanceXsec=enhanceXsec;
	BHparticle::TotalDim=TotalDim;
	BHparticle::MJlostGraviton=MJlostGraviton;
	BHparticle::cvbias=cvbias;
	ifstream yosh_datafile;		//Yoshino-Rychkov data files
	switch (TotalDim){
	case (5): yosh_datafile.open("Charybdisdata/MLB_N1.data");break;
	case (6): yosh_datafile.open("Charybdisdata/MLB_N2.data");break;
	case (7): yosh_datafile.open("Charybdisdata/MLB_N3.data");break;
	case (8): yosh_datafile.open("Charybdisdata/MLB_N4.data");break;
	case (9): yosh_datafile.open("Charybdisdata/MLB_N5.data");break;
	case (10):yosh_datafile.open("Charybdisdata/MLB_N6.data");break;
	case (11):yosh_datafile.open("Charybdisdata/MLB_N7.data");break;
	default:cerr<<"Error: TotalDim value not suported by currently encoded Yoshino-Rychkov files"<<endl;
			exit(EXIT_FAILURE);
	}
	if (!yosh_datafile.is_open()){        //tests if the files was correctly opened
		cerr<<"Error: Unable to open the Yoshino-Rychkov data file."<<endl;   //display error message
		exit(EXIT_FAILURE); //and terminate the program
	}
	double yosh_data;
	while (yosh_datafile>>yosh_data && !yosh_datafile.eof()){
		BHparticle::MJlostdata_b.push_back(yosh_data);
		yosh_datafile>>yosh_data;
		BHparticle::MJlostdata_Mb.push_back(yosh_data);
	}
	if (BHparticle::MJlostdata_b.size()!=BHparticle::MJlostdata_Mb.size()){
		cerr<<"Error: Yoshino-Rychkov data file apears to be corrupted."<<endl;
		exit(EXIT_FAILURE);
	}
	yosh_datafile.close();
}

void BHgenerator::CharybdisInit(){
  /********computes the function the maximum (LH_head.xmaxup), the average (LH_head.xsecup), and the error margin (LH_head.xerrup) and stores it in the LH_block********/              
  double chwsum=0.0,chwsq=0.0;        									//weight sum and squared weight sum
  LH_head.xmaxup=0;                    									//max 
  for (int i=0;i<mc_samplesize;++i) {              						//for the selected number of events
    double xwgtup=CharybdisProduction(minBHmass,maxBHmass);  			//generate a random weighted event acording to distribution of the function called
    Event.reset();
    chwsum+=xwgtup;                         							//sum the weights
    chwsq+=xwgtup*xwgtup;                   							//and the squared weights
    if (xwgtup>LH_head.xmaxup)                      					//if the generator is higher than the recorded maximum
      LH_head.xmaxup=xwgtup;                      						//it becomes the new recorded maximum
    }    
    LH_head.xsecup=chwsum/mc_samplesize;                                                   			//calculate the average
    LH_head.xerrup=sqrt((chwsq/mc_samplesize-LH_head.xsecup*LH_head.xsecup)/mc_samplesize);      	//and calculate the error threw stardart deviation
	cout<<"TotalDim: "<<TotalDim<<"\tPlanckMass: "<<PlanckMass<<"\tminBHmass: "<<minBHmass<<endl;
	cout<<"cross-section: "<<LH_head.xsecup<<" +- "<<LH_head.xerrup<<" pb"<<endl;
}

double BHgenerator::diffXsectionBin(double MinEcmPartonic,double MaxEcmPartonic,int nweigths){              
  double chwsum=0.0;       												//weight sum and squared weight sum
  for (int i=0;i<nweigths;++i) {              							//for the selected number of events
    double xwgtup=CharybdisProduction(MinEcmPartonic,MaxEcmPartonic);  	//generate a random weighted event acording to distribution of the function called
    Event.reset();
    chwsum+=xwgtup;                         							//sum the weights
  }    
    return chwsum/nweigths/(MaxEcmPartonic-MinEcmPartonic);             //calculate the average
}
 
void BHgenerator::diffXsectionBin(int num_bins,int samplesize,string outputstr){
/********calculates the deferential cross-section ********/
  ofstream testXsection(outputstr.c_str());
  double difxsecstep=(maxBHmass-minBHmass)/num_bins;
  testXsection<<"lb_mass\tdiffXsection"<<endl;
  for (double m=minBHmass;m<maxBHmass;m=m+difxsecstep)														//use this funcion as:
	testXsection<<m<<"\t"<<samplesize*diffXsectionBin(m,m+difxsecstep,samplesize)<<endl;  		//diffxsecstep(bin_min,bin_max,sample_perbin)
}
 
   
void BHgenerator::define_constants(){
  /******defines internal variables to be used later on******/  
  pi=acos(-1);                //defines pi
  nex=TotalDim-4;             //defines nex as the number of extra dimensions
  /********definition of planck mass*************/
  if (planckmassdef==1)
    intmpl=PlanckMass;
  else if (planckmassdef==2)
    intmpl=PlanckMass*pow(pow(2,TotalDim-6)*pow(pi,TotalDim-5),1/(TotalDim-2.0));
  else if (planckmassdef==3)
    intmpl=PlanckMass*pow(2,1/(TotalDim-2.0));
  else if (planckmassdef==4)
    intmpl=customPlanckmassdef(PlanckMass,TotalDim);                 
  else {
    cerr<<endl<<"Error: planckmassdef was not correctly defined."<<endl;
    exit(EXIT_FAILURE);
  }
  /*********defines several variables to avoid repeating calculations inside the cycles**********/
  PLpow=2/(TotalDim-3.0);                                     //
  BHpow=PLpow-7.0;                                            //
  Rpow=1/BHpow;                                               //
  mpfact=pow(intmpl,TotalDim-2);                              //
  omegad=nsphere_area_factor(TotalDim-1);                     //
  RHfact=4*pow(2*pi,TotalDim-4)/((TotalDim-2)*mpfact*omegad); //
  Gev2pb=3.89379e8;                                           //GeV to picobarn conversion
}

double BHgenerator::CharybdisProduction(double MinEcmPartonic,double MaxEcmPartonic){
  
	double Ecms=2*sqrt(LH_head.Ebmup[0]*LH_head.Ebmup[1]);                      		//center of mass energy
	double Amin=pow(MinEcmPartonic,BHpow);                                    			//
	double Amax=pow(MaxEcmPartonic,BHpow);                                     			//
	double Emsca=pow(Amin+(Amax-Amin)*rand.Random(),Rpow);              				//generate the energy scale
	double BHmass=Emsca;
	double RH_sq=pow(RHfact*BHmass,PLpow);
	double xxmin=pow(Emsca,2)/pow(Ecms,2);
	double xlogmin=log(xxmin);
	double fact=-2*pi*Gev2pb*RH_sq*xlogmin*pow(Emsca,1-BHpow)*(Amax-Amin)/(BHpow*Emsca);     
	/***Aplying cross-section enhancement****/
	double YoshRychfactor[7]={1.54,2.15,2.52,2.77,2.95,3.09,3.20};		//yoshino-Rychkov cross section enhancment factor dependent on D
	if (enhanceXsec==1)													//see Phys.Rev.D71:104028,2005 [hep-th/0503171]
		fact=fact*YoshRychfactor[TotalDim-5];
	else if (enhanceXsec==2)
		fact=customXsec(fact,TotalDim);
	else if (enhanceXsec!=0){
		cerr<<"Error: enhanceXsec was given an invalid value."<<endl;
		exit(EXIT_FAILURE);
	}
	/**Change momentum scale for calling pdfs**/
	if (GTscale)
		Emsca=1/sqrt(RH_sq);
	scalup=Emsca;               					
	pdfholder.callPDFs(xxmin,xlogmin,Emsca,rand);		//call the pdfs from LHAPDF
	double hcs=0,disf1=0,disf2=0;
	for (int i=0;i<13;++i){								//
		disf1+=pdfholder.PDFx1f1[i];
		disf2+=pdfholder.PDFx2f2[i];
	} 
	double factr=fact*disf1*disf2;						//calculates xweight as the sum of the weights of the pairs of partons
	hcs=factr;
	double xweight=hcs;
	if (MJlostGraviton && useminBHmass && !SelectIncomingParticle(hcs,fact)) {	//if the MJlost rejection is selected the full production
		Event.reset();															//needs to be processed 
		return 0.0;
	}
	else
		return xweight;
}

bool BHgenerator::SelectIncomingParticle(double HCStotal,double fact){
/*********selects incoming particles from the PDFs**********/
	double rcs=HCStotal*rand.Random();
	double hcs=0,factr=0;
	int part_id[2];										//<------colision particle id's will be store here
	for (part_id[0]=0;part_id[0]<13;++part_id[0]){
		bool breaker=false;
		for (part_id[1]=0;part_id[1]<13;++part_id[1]){										//select the incoming particles
			factr=fact*pdfholder.PDFx1f1[part_id[0]]*pdfholder.PDFx1f1[part_id[1]];			//using monte carlo
			hcs+=factr;
			if (hcs>rcs){
				breaker=true;
				break;
			}
		}
		if (breaker)
			break;
	}
	/****switching particle id numering scheme to les houches ****/
	for (int i=0;i<2;++i) {										//for both particles.....
		if (part_id[i]==13)										//quarks:			1, 2, 3, 4, 5, 6  ------>  1, 2, 3, 4, 5, 6	
			part_id[i]=21;										//gluon: 			6---> 21
		else if (part_id[i]>6 && part_id[i]<=12)				//anti-quarks: 		7, 8, 9,10,11,12  ------> -1,-2,-3,-4,-5,-6
			part_id[i]=-(part_id[i]-6);																			
	}
	/**************creating particles*****************/
	Event.part.push_back(particle(part_id[0],pdfholder.x1*LH_head.Ebmup[0]));
	Event.part.push_back(particle(part_id[1],-pdfholder.x2*LH_head.Ebmup[1]));	//creates the particles
	/**********momentum fraction warning**************/
	if (Event.part[0].Pup[3]>LH_head.Ebmup[0] || Event.part[1].Pup[3]>LH_head.Ebmup[1]){	//if the energy of the particles is  greater than
		clog<<"Warning: A paricle was created with energy superior to it's beam."<<endl		//the energy of the beam: display a warning
			<< "Id= "<<Event.part[1].IDup<<"\tPz= "<<Event.part[1].Pup[3]<<"\tEbmup= "<<LH_head.Ebmup[1]<<endl
			<<"This result will be ignored."<<endl
			<<"If you receive too many of this messages please check your minBHmass & maxBHmass parameters."<<endl;
		return false;
	}
	/****************creating black hole*************/
	Event.blackhole.push_back(BHparticle(Event.part[0],Event.part[1]));		//creates the 1st black hole
	if (Event.blackhole[0].Pup[4]<minBHmass)
		return false;
	else
		return true;
}

