#ifndef CHARYBDIS_BHGENERATOR_H
#define CHARYBDIS_BHGENERATOR_H
#include <istream>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>
#include <cmath>
#include "randomc.h"
#include "charybdis_ccinterface.h"
#include "charybdis_tools.h"
#include "charybdis_PDFs.h"
#include "charybdis_user.h"
#include "charybdis_event.h"
using namespace std;

class BHgenerator {
public:
  /******************* CONSTRUCTOR *********************/
  BHgenerator(ccinterface&,CRandomMersenne&);  
  friend class ccinterface;
  /*************** PUBLIC MEMBER FUNCTIONS *************/
  void CharybdisInit(); // Performs weight search and computes cross section
  double diffXsectionBin(double,double,int);   
  void diffXsectionBin(int,int,string);   
  double CharybdisProduction(double,double);
    
private:
  /********************internal data*********************/  
  LH_block LH_head; 
  PDFs pdfholder;
  CRandomMersenne rand;
  event Event;
  int mc_samplesize;
  double pi;
  int nex;
  double intmpl;
  double PLpow;
  double BHpow;
  double Rpow;
  double mpfact;
  double omegad;
  double RHfact;
  double Gev2pb;
  double scalup;
  vector<int> hist;   //just used for test_histogram
  /********************File streams*********************/
  ofstream &lhevent,&eventhistory;
  /***********************Input data******+************/
  double minBHmass;           	//minimum BH mass accepted during production (in Gev)
  double maxBHmass;           	//maximum BH mass c during production (in Gev)
  int NumMaxEvents;           	//number of events to generate
  bool BH_in_LHE;             	//defines if the black hole is appears as an intermediary particle on the Les Houches file or not
  int verbositycode;          	//defines the amount of information to output to cout
  int TotalDim;               	//number of dimensions to use
  double PlanckMass;          	//planck mass to use (in Gev)
  int planckmassdef;          	//switch that defines how planck's mass is calculated
  int enhanceXsec;          	//switch that defines the use Yoshino-Rychkov impact cross-section calculation
  bool MJlostGraviton;        	//switch that defines if there is mass loss threw gravity waves in the balding phase
  bool useminBHmass;          	//switch that defines if there is a min BH mass at evaporation or if it goes down to planck scale
  bool cvbias;                	//switch that defines the use of the constant angular velocity bias
  double isotroploss;         	//loss of isotropy threw gravitational radiation
  bool GTscale;               	//switch that defines the use of Giddings-Thomas scale for the PDF's
  bool nleptonicsv[4];        	//switchs that define if TOTAL leptonic number[0], leptonic [1], muonic  [2] and tau [3] conservation is imposed
  bool BHspin;                	//switch that defines if the black hole has angular momentum
  bool BHspinvar;             	//switch that defines if the angular momentum may vary during the evaporation phase
  bool BHanis;                	//switch that defines the use non-uniform angular functions during evaporation
  bool greybody;              	//switch that defines the use of grey-body factors
  bool THvar;                 	//switch that defines if the Hawking's temperature may vary during evaporation
  int alloweddecay;           	//switch that defines which particles may be produced during evaporation
  int BHrecoil;               	//switch that defines the BH recoil model
  double THmax;               	//maximum Hawking's temperature allowed
  bool nbodyaverage;          	//switch that defines if the average expected particle number should be used to terminate evaporation
  bool kincut;                	//switch that defines if the kinematic cut-off should be used to terminate evaporation
  bool skip2remnant;          	//switch used to skip the evaporation phase directly to the remnant
  int RMdecaynum;             	//defines the number of particles the remnant decays to
  bool RMphasedecay;          	//switch that defines if remnant decay uniform ("phase-wise") or depends on remnant spin
  bool RMvarMultiplicity;     	//switch that defines the use of the variable-multiplicity remnant model or not
  bool RMboil;                	//switch that defines the use of the boiling remnant model or not
  bool RMstable;              	//switch that defines the use of the stable remnant model or not
  double RMminmass;           	//remnant minimum mass for the boiling model
  /******************Member functions******************/
  void define_constants();            	//defines constants for internal use
  void initclass(ccinterface&);
  void printruninfo(); 					//Prints onscreen information about the program and run conditions
  bool SelectIncomingParticle(double,double);
  void initBHstatic();
};

#endif
