#include "charybdis_ccinterface.h"

/******************CLASS CONSTRUCTOR******************/
ccinterface::ccinterface(int argc, char *argv[]):argnumber(argc-1),endinput("---END OF INPUT---") {
  /*storing bufs in case they are redirected*/
  coutbuf = cout.rdbuf();         //cout original buf
  clogbuf = clog.rdbuf();         //clog original buf
  cerrbuf = cerr.rdbuf();         //cerr original buf
  /*************check console commands**********/
  checkhelp(argc,argv);           //checks if --help was called
  model_file(argc,argv);
  open_input(argc,argv,"-i");     //opens the input file named as the string after "-i"
  checkfile(input,"input");       //checks if the file was opened correctly
  /********redirect cout, clog & cerr if asked to********/
  rdcout=RedirectOstream(cout,newcout,argc,argv,"-o");    //redirects cout if the command was given
  if (rdcout)                             //if cout was redirected
    checkfile(newcout,"cout redirect"); //checks if the file was opened correctly
  rdclog=RedirectOstream(clog,newclog,argc,argv,"--log"); //redirects log if the command was given
  if (rdclog)                             //if clog was redirected
    checkfile(newclog,"clog redirect"); //checks if the file was opened correctly
  rdcerr=RedirectOstream(cerr,newcerr,argc,argv,"--err"); //redirects cerr if the command was given
  if (rdcerr)                             //if cerr was redirected
    checkfile(newcerr,"cerr redirect"); //checks if the file was opened correctly
  open_output(argc,argv,lhevent,"--lhevent","LH_output.xml"); //opens the lhevent file (uses the last string has default if not specified
  checkfile(lhevent,"Les Houches event"); //checks if the file was opened correctly
  open_output(argc,argv,eventhistory,"--eventhist","Event_history.xml");  //opens the event history file (uses the last string has default if not specified
  checkfile(eventhistory,"event history");//checks if the file was opened correctly
  if (argnumber!=0){
	  cout<<"Error: Invalid arguments in the command console."<<endl;
	  exit(EXIT_FAILURE);
  }	
  lhevent.precision(8);
  lhevent<<scientific;
  eventhistory.precision(8);
  eventhistory<<scientific;				//sets the precision of the floating points for the output file
  input_and_headers();                      //extracts the input data and initalizes the headers for the output files
}

int ccinterface::seed() {
  //returns the seed for the random number generator
  return randseed;
}

void ccinterface::checkhelp(int argc, char*argv[]){
  //checks if the help command was given
  for (int i=1;i<argc;++i){   //for all console commands
    string s=argv[i];       //s=console command
    if (s=="--help") {      //if s=--help
      help();             //print the help instructions
      exit(EXIT_SUCCESS);
    }
  }
}

void ccinterface::model_file(int argc,char *argv[]) {
  //checks if the --model command was given and
  //creates a model input.init file if it was
  for (int i=1;i<argc;++i){   //for all console commands
    string s=argv[i];        //s=console command
    if (s=="--model") {      //if s=--model
      print_model();       //print the help instructions
      cout<<"A model input file (input.init) was created in this directory."<<endl;
      exit(EXIT_SUCCESS);
    }
  }
}


void ccinterface::help() {
  //print the list of commands
  cout<<endl
      <<"This program is intended to be run primarily from the command console."<<endl
      <<"You should run it as \"charybdisgen -i [inputfile] [aditional commands]\"."<<endl
      <<endl
      <<"Additional specifications may be applied using the following commands:"<<endl
      <<endl
      <<"--lhevent [file.xml] determines a new path to the creation of the les houches"<<endl
      <<"file (by default it will be created a file named LH_output.xml"<<endl
      <<"in this directory)."<<endl
      <<endl
      <<"--eventhist [file.txt] determins a new path to the creation of the"<<endl
      <<"event history file (by default it will be created a file"<<endl
      <<"named Event_history.txt in this directory)."<<endl
      <<endl
      <<"-o [file] redirects the cout stream to the file path given."<<endl
      <<endl
      <<"--log [file] redirects the clog stream to the file path given."<<endl
      <<endl
      <<"--err [file] redirects the cerr stream to the file path given."<<endl
      <<endl
      <<"--model creates a model input.init file in this directory."<<endl
      <<endl
      <<"--help brings up this instruction set."<<endl<<endl;
}

void ccinterface::open_input(int argc, char*argv[], string option){
  //opens the input file from the string given after the option on the command console
  //if no file path was given call no_cc_msg to prompt the user for a file path
  string input_str;
  for (int i=1;i<argc-1;++i)  {   //for all console commands except the last
    string s=argv[i];                  //s=console command
    if (s==option)    {         //if s is the given command (option)
      input_str=argv[i+1];    //the string given after the command "s" is the input file name
      input.open(input_str.c_str());  //open the input file
      argnumber+=-2;
      break;
    }
  }
  if (!input.is_open())     //if no input command was found
    no_cc_msg();            //call no_cc_msg to prompt the user for it
}

void ccinterface::no_cc_msg(){
  //if no commands were given explains how to use console commands
  // and asks the user for the input file
  help();
  cout<<"Alternatively you may define the input file here (all other specifications"<<endl
      <<"will be regarded has defaulf if you have not defined them):"<<endl;
  string input_str;
  cin>>input_str;                 //gets the input file name from the user
  input.open(input_str.c_str());  //open the inputfile with that name
}

bool ccinterface::RedirectOstream(ostream& os,ofstream& newOstream,int argc,char *argv[],string option){
  //checks if a command (option) was given in the console.
  //if it was, opens an ofstream with the path given and redirects the ostream to it
  //returns true if there was a redirection and false when it does nothing
  for  (int i=1;i<argc-1;++i){    //for all console commands except the last
    string s=argv[i];           //s=console command
    if (s==option){             //if s is the given command (option)
      string filename=argv[i+1];          //filename is the string given after the command "s"
      newOstream.open(filename.c_str());  //open a stream with that name
      os.rdbuf(newOstream.rdbuf());       //redirect the ostream to it (cout, clog or cerr)
      argnumber+=-2;					  //and decreases the argument number counter
      return true;
    }
  }
  return false;
}

bool ccinterface::open_output(int argc,char*argv[],ofstream& output,string option,string default_str){
  //checks if a file path to one of the output files was given to the console.
  //if it was assigns it to output, if it wasn't assigns default_str instead
  //returns 0 if the default_str was used and 1 otherwise
  for (int i=1;i<argc-1;++i){             //for all console commands except the last
    string s=argv[i];                   //s=console command
    if (s==option){                     //if s is the given command (option)
      string filename=argv[i+1];      //filename is the string given after the command "s"
      output.open(filename.c_str());  //open the outputfile with that name with that name
      argnumber+=-2;					  //and decreases the argument number counter
      return true;
    }
  }
  output.open(default_str.c_str());   //if no command was found use default_str to open it
  return false;
}

void ccinterface::resetbufs(){
  //resets the output bufs
  if (rdcout)                 //if the cout was redirected
    cout.rdbuf(coutbuf);    //resets cout
  if (rdclog)                 //if the clog was redirected
    clog.rdbuf(clogbuf);    //resets clog
  if (rdcerr)                 //if the cerr was redirected
    cerr.rdbuf(cerrbuf);    //resets cerr
}

void ccinterface::checkfile(ifstream&  ifile,string name){
  //checks if the input file was opened correctly
  if (!ifile.is_open()){        //if it wasn't
    cerr<<"Error: Unable to open the "<<name<<" file."<<endl;   //display error message
    exit(EXIT_FAILURE); //and terminate the program
  }
}

void ccinterface::checkfile(ofstream&  ofile,string name){
  //checks if the output file was opened correctly
  if (!ofile.is_open()){        //if it wasn't
    cout<<"Error: Unable to open the "<<name<<" file."<<endl;   //display error messages
    cerr<<"Error: Unable to open the "<<name<<" file."<<endl;   //display error messages
    exit(EXIT_FAILURE); //and terminate the program
  }
}

void ccinterface::outputheaderinfo(ofstream& outfile){
  //creates the header for the output files
  outfile<<"<!--"<<endl
	 <<"File generated with charybdis2cpp v.(...)"<<endl
	 <<"WEBSITE PLACEHOLDER"<<endl
	 <<endl
	 <<"Bellow are the variables and switches used in the run."<<endl
	 <<"You may rerun by copying them (including the end of input line) to a file and using it as input."<<endl
	 <<"A model .init file was provided with this distribuition containing instructions on how to use each variable."<<endl
	 <<"Please read that file for instructions on how to control your runs."<<endl
	 <<endl
	 <<"---INPUT OPTIONS:"<<endl;
}

void ccinterface::input_and_headers(){
  //reads the input file and calls getinput() to extract every parameter for the generator
  //and creates the headers for the output files
  lhevent<<"<LesHouchesEvents version=\"1.0\">"<<endl;    //initialises the header
  eventhistory<<"<BHhistories>"<<endl;                    //on both output files
  outputheaderinfo(lhevent);                              //prints out some information
  outputheaderinfo(eventhistory);                         //on both headers
  checkinput(LH_head.IDbmup[0],"IDbmup[0]:");
  checkinput(LH_head.IDbmup[1],"IDbmup[1]:");
  checkinput(LH_head.Ebmup[0],"Ebmup[0]:");
  checkinput(LH_head.Ebmup[1],"Ebmup[1]:");
  checkinput(minBHmass,"minBHmass:");
  checkinput(maxBHmass,"maxBHmass:");
  checkinput(LH_head.PDFname[0],"PDFname[0]:");
  checkinput(LH_head.PDFname[1],"PDFname[1]:");
  checkinput(LH_head.PDFSup[0],"PDFSup[0]:");
  checkinput(LH_head.PDFSup[1],"PDFSup[1]:");
  checkinput(LH_head.PDFnum[0],"PDFnum[0]:");
  checkinput(LH_head.PDFnum[1],"PDFnum[1]:");
  checkinput(NumMaxEvents,"NumMaxEvents:");
  checkinput(mc_samplesize,"mc_samplesize:");
  checkinput(randseed,"randseed:");
  checkinput(BH_in_LHE,"BH_in_LHE:");
  checkinput(verbositycode,"verbositycode:");
  checkinput(TotalDim,"TotalDim:");
  checkinput(PlanckMass,"PlanckMass:");
  checkinput(planckmassdef,"planckmassdef:");
  checkinput(enhanceXsec,"enhanceXsec:");
  checkinput(MJlostGraviton,"MJlostGraviton:");
  checkinput(useminBHmass,"useminBHmass:");
  checkinput(cvbias,"cvbias:");
  checkinput(isotroploss,"isotroploss:");
  checkinput(GTscale,"GTscale:");
  checkinput(nleptonicsv[0],"nleptonicsv[0]:");
  checkinput(nleptonicsv[1],"nleptonicsv[1]:");
  checkinput(nleptonicsv[2],"nleptonicsv[2]:");
  checkinput(nleptonicsv[3],"nleptonicsv[3]:");
  checkinput(BHspin,"BHspin:");
  checkinput(BHspinvar,"BHspinvar:");
  checkinput(BHanis,"BHanis:");
  checkinput(greybody,"greybody:");
  checkinput(THvar,"THvar:");
  checkinput(alloweddecay,"alloweddecay:");
  checkinput(BHrecoil,"BHrecoil:");
  checkinput(THmax,"THmax:");
  checkinput(nbodyaverage,"nbodyaverage:");
  checkinput(kincut,"kincut:");
  checkinput(skip2remnant,"skip2remnant:");
  checkinput(RMdecaynum,"RMdecaynum:");
  checkinput(RMphasedecay,"RMphasedecay:");
  checkinput(RMvarMultiplicity,"RMvarMultiplicity:");
  checkinput(RMboil,"RMboil:");
  checkinput(RMstable,"RMstable:");
  checkinput(RMminmass,"RMminmass:");
  input.close();
  lhevent<<endl<<endinput<<endl<<endl<<"-->"<<endl<<endl;       //finish the header comments
  eventhistory<<endl<<endinput<<endl<<endl<<"-->"<<endl<<endl;  //finish the header comments
}

void ccinterface::close_all_streams(){
  /******close all output streams*******/
  lhevent.close();
  eventhistory.close();
  if (rdcout)
    newcout.close();
  if (rdclog)
    newclog.close();
  if (rdcerr)
    newcerr.close();
}

void ccinterface::checkinput(int& variable,string name){
  //reads the input file until if finds the parameter "name" and then
  //extracts the value bellow it in the given variable
  //and prints the information on the output file headers
  string reader;
  while(reader!=endinput && reader!=name) //while reader doesnt find the variable in the input file or reach the end of input
    getline(input,reader);              //it reads the input file line by line
  if (reader==endinput) {                 //if it didn't find the intended variable name
    cerr<<"Error: Unable to find the "<<name<<" parameter in the input file."<<endl;    //it outputs an error message
    exit(EXIT_FAILURE);                 //and terminates the program
  }
  lhevent<<reader<<endl;                   //if it does find the parameter
  eventhistory<<reader<<endl;              //it prints it's name on the output files
  input>>variable;                        //then it extracts the variable value from the next line
  lhevent<<variable<<endl;                //and prints it
  eventhistory<<variable<<endl;           //on both files
  input.seekg(0,input.beg);               //and resets the file stream back to the top
}

void ccinterface::checkinput(double& variable,string name){
  //reads the input file until if finds the parameter "name" and then
  //extracts the value bellow it in the given variable
  //and prints the information on the output file headers
  string reader;
  while(reader!=endinput && reader!=name) //while reader doesnt find the variable in the input file or reach the end of input
    getline(input,reader);              //it reads the input file line by line
  if (reader==endinput) {                 //if it didn't find the intended variable name
    cerr<<"Error: Unable to find the "<<name<<" parameter in the input file."<<endl;    //it outputs an error message
    exit(EXIT_FAILURE);                 //and terminates the program
  }
  lhevent<<reader<<endl;                   //if it does find the parameter
  eventhistory<<reader<<endl;              //it prints it's name on the output files
  input>>variable;                        //then it extracts the variable value from the next line
  lhevent<<variable<<endl;                //and prints it
  eventhistory<<variable<<endl;           //on both files
  input.seekg(0,input.beg);               //and resets the file stream back to the top
}

void ccinterface::checkinput(string& variable,string name){
  //reads the input file until if finds the parameter "name" and then
  //extracts the value bellow it in the given variable
  //and prints the information on the output file headers
  string reader;
  while(reader!=endinput && reader!=name) //while reader doesnt find the variable in the input file or reach the end of input
    getline(input,reader);              //it reads the input file line by line
  if (reader==endinput) {                 //if it didn't find the intended variable name
    cerr<<"Error: Unable to find the "<<name<<" parameter in the input file."<<endl;    //it outputs an error message
    exit(EXIT_FAILURE);                 //and terminates the program
  }
  lhevent<<reader<<endl;                   //if it does find the parameter
  eventhistory<<reader<<endl;              //it prints it's name on the output files
  input>>variable;                        //then it extracts the variable value from the next line
  lhevent<<variable<<endl;                //and prints it
  eventhistory<<variable<<endl;           //on both files
  input.seekg(0,input.beg);               //and resets the file stream back to the top
}

void ccinterface::checkinput(bool& variable,string name){
  //reads the input file until if finds the parameter "name" and then
  //extracts the value bellow it in the given variable
  //and prints the information on the output file headers
  string reader;
  while(reader!=endinput && reader!=name) //while reader doesnt find the variable in the input file or reach the end of input
    getline(input,reader);              //it reads the input file line by line
  if (reader==endinput) {                 //if it didn't find the intended variable name
    cerr<<"Error: Unable to find the "<<name<<" parameter in the input file."<<endl;    //it outputs an error message
    exit(EXIT_FAILURE);                 //and terminates the program
  }
  lhevent<<reader<<endl;                   //if it does find the parameter
  eventhistory<<reader<<endl;              //it prints it's name on the output files headers
  input>>variable;                         //then it exctracts the variable value from the next line
  lhevent<<variable<<endl;                 //and prints it on the header of both output files
  eventhistory<<variable<<endl;
  input.seekg(0,input.beg);               //if no errors were found it resets the file stream back to the top
}

void ccinterface::print_model(){
  //prints the model input.init file
  ofstream model_file;
  model_file.open("input.init");
  model_file<<"*************************************************************"<<endl
	    <<"*************************************************************"<<endl
	    <<"**************  CHARYBDIS2CPP RUN PARAMETERS  ***************"<<endl
	    <<"*************************************************************"<<endl
	    <<"*************************************************************"<<endl
	    <<""<<endl
	    <<"INSTRUCTIONS/DESCRIPTION (PLEASE READ CAREFULLY!!!):"<<endl
	    <<""<<endl
	    <<"Below is a list of all the input parameters and switches that may be changed from the internal default values in charybdis++."<<endl
	    <<"This has been divided into logical sections to facilitate the identification of the parameter you want to change from default."<<endl
	    <<""<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<"!!!! DO NOT CHANGE OR DELETE THE VARIABLE NAMES DEFINED !!!!!"<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<""<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<"!!!!!!! DO NOT CREATE LINES BETWEEN VARIABLE NAMES !!!!!!!!!!"<<endl
	    <<"!!!!!!!              AND THEIR VALUE               !!!!!!!!!!"<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<""<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<"!!!!!!!! DO NOT WRITE THE VARIABLE NAMES ANYWHERE !!!!!!!!!!!"<<endl
	    <<"!!!!!!!! IN THIS DOCUMENT (you may write them in  !!!!!!!!!!!"<<endl
	    <<"!!!! !!! comments on empty lines provided you use !!!!!!!!!!!"<<endl
	    <<"!!!!!!!!            different Captation)          !!!!!!!!!!!"<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<""<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<"!!!!!!!!            DO NOT REMOVE THE             !!!!!!!!!!!"<<endl
	    <<"!!!!!!!!           '---END OF INPUT---'           !!!!!!!!!!!"<<endl
	    <<"!!!!!!!!                   LINE                   !!!!!!!!!!!"<<endl
	    <<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"***************** START OF INPUT OPTIONS ********************"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"------------ Beams & Energies --------------"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"Beam 1 particle identification number (IDB)."<<endl
	    <<"Default is 2212 (proton)."<<endl
	    <<"IDbmup[0]:"<<endl
	    <<"2212"<<endl
	    <<""<<endl
	    <<"Beam 2 particle identification number (IDB)."<<endl
	    <<"Default is 2212 (proton)."<<endl
	    <<"IDbmup[1]:"<<endl
	    <<"2212"<<endl
	    <<""<<endl
	    <<"Beam 1 energy (Gev)."<<endl
	    <<"Default is 7000.0."<<endl
	    <<"Ebmup[0]:"<<endl
	    <<"7000.0"<<endl
	    <<""<<endl
	    <<"Beam 2 energy (Gev)."<<endl
	    <<"Default is 7000.0."<<endl
	    <<"Ebmup[1]:"<<endl
	    <<"7000.0"<<endl
	    <<""<<endl
	    <<"Minimum black hole mass accepted during production (Gev)."<<endl
	    <<"Default is 5000.0."<<endl
	    <<"minBHmass:"<<endl
	    <<"5000.0"<<endl
	    <<""<<endl
	    <<"Maximum black hole mass accepted during production (Gev)."<<endl
	    <<"Default is 14000.0."<<endl
	    <<"maxBHmass:"<<endl
	    <<"14000.0"<<endl
	    <<""<<endl
	    <<"PDF group to use from the LHAPDF library for beam 1."<<endl
	    <<"Default is CT10"<<endl
	    <<"PDFname[0]:"<<endl
	    <<"CT10"<<endl
	    <<""<<endl
	    <<"PDF group to use from the LHAPDF library for beam 2."<<endl
	    <<"Default is CT10"<<endl
	    <<"PDFname[1]:"<<endl
	    <<"CT10"<<endl
	    <<""<<endl
	    <<"PDF set to use from the LHAPDF library for beam 1."<<endl
	    <<"Default is 0"<<endl
	    <<"PDFSup[0]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"PDF set to use from the LHAPDF library for beam 2."<<endl
	    <<"Default is 0"<<endl
	    <<"PDFSup[1]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"PDF member number from the LHAPDF library for beam 1."<<endl
	    <<"Recommended value is 0 (LHAPDF defines 0 as the central distribution)."<<endl
	    <<"PDFnum[0]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"PDF member number from the LHAPDF library for beam 2."<<endl
	    <<"Recommended value is 0 (LHAPDF defines 0 as the central distribution)."<<endl
	    <<"PDFnum[1]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"---------------- MC & OUTPUT ---------------"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"Number of events to generate."<<endl
	    <<"NumMaxEvents:"<<endl
	    <<"1000"<<endl
	    <<""<<endl
	    <<"Sample number for the Monte Carlo generator."<<endl
	    <<"Default is 10000."<<endl
	    <<"mc_samplesize:"<<endl
	    <<"10000"<<endl
	    <<""<<endl
	    <<"Random number generator Seed."<<endl
	    <<"randseed:"<<endl
	    <<"299792458"<<endl
	    <<""<<endl
	    <<"Switch defining weather or not to use the black hole as an intermediary particle in the Les Houches file (T or F)."<<endl
	    <<"BH_in_LHE:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Verbosity setting for the amount of information displayed on the terminal. "<<endl
	    <<"Default is 1"<<endl
	    <<"verbositycode:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"----- Model parameters and conventions -----"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"Total number of dimensions to use."<<endl
	    <<"Default is 6."<<endl
	    <<"TotalDim:"<<endl
	    <<"6"<<endl
	    <<""<<endl
	    <<"Plank Mass to use (Gev)."<<endl
	    <<"Default is 10000.0"<<endl
	    <<"PlanckMass:"<<endl
	    <<"10000.0"<<endl
	    <<""<<endl
	    <<"Planck's Mass dependence on D model. chose 1 for Giddings-Thomas; 2 for Dimopolous-Landsberg; 3 for PDG and 4 for a user defined equation (that needed to be programed in the 'user.cpp' file)."<<endl
	    <<"planckmassdef:"<<endl
	    <<"3"<<endl
	    <<""<<endl
	    <<"Cross-section enhancement switch."<<endl
	    <<"Use '0' for no enhancement, '1' for the Yoshino-Rychkov model and '2' to use a costume fuction (that you'll need to implement in the 'charybdis_user.cpp' file)."<<endl
	    <<"(Default is 1)"<<endl
	    <<"enhanceXsec:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Switch defining whether or not to simulate loss of mass and angular momentum threw gravitational waves during the production phase."<<endl	    
	    <<"Warning: it makes no sense setting this parameter to true if you are not using an enhanced cross-section."<<endl
	    <<"Set to 1 or 0 (True or False)."<<endl
	    <<"MJlostGraviton:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Switch that defines whether the BH only evaporates to a set minimum mass (1) or if it is allowed to evaporate until it reachs the Planck Scale (0)."<<endl
	    <<"useminBHmass:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Switch defining the use of 'constant angular velocity bias' (1 or 0)."<<endl
	    <<"cvbias:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Loss of isotropy of the gravitational radiation."<<endl
	    <<"(Default is 0.99)"<<endl
	    <<"isotroploss:"<<endl
	    <<"0.99"<<endl
	    <<""<<endl
	    <<"Use Giddings-Thomas scale for the PDF's (1 or 0)."<<endl
	    <<"GTscale:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Conservation of the TOTAL lepton number (1 or 0)."<<endl
	    <<"nleptonicsv[0]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Conservation of the lepton number (1 or 0)."<<endl
	    <<"nleptonicsv[1]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Conservation of the muon number (1 or 0)."<<endl
	    <<"nleptonicsv[2]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Conservation of the tau number (1 or 0)."<<endl
	    <<"nleptonicsv[3]:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"------------Evaporation Switches------------"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"The BH is permited to have spin (1 or 0)."<<endl
	    <<"BHspin:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Allow the BH spin to vary during evaporation (1 or 0)."<<endl
	    <<"BHspinvar:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Allow Non-uniform angular functions for the evaporation (1 or 0)."<<endl
	    <<"BHanis:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Include grey-body factors (1 or 0)."<<endl
	    <<"greybody:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Allow Halking's temperature to vary during evaporation (1 or 0)."<<endl
	    <<"THvar:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Switch defining which particles may be emitted during evaporation."<<endl
	    <<"1 allows '...'; 2 allows '...'; 3 allows all particles of the standard model and 4 allows a user defined set of particles."<<endl
	    <<"To use option 4 you will have to edit the distribution function in the user.cpp file."<<endl
	    <<"(Default is 3)"<<endl
	    <<"alloweddecay:"<<endl
	    <<"3"<<endl
	    <<""<<endl
	    <<"Switch defining the BH recoil model."<<endl
	    <<"(Default is 2)"<<endl
	    <<"BHrecoil:"<<endl
	    <<"2"<<endl
	    <<""<<endl
	    <<"Maximum Hawking temperature the BH is allowed to reach (Gev)."<<endl
	    <<"(Default is 1000.0)"<<endl
	    <<"THmax:"<<endl
	    <<"1000.0"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"--- Switches to terminate the evaporation --"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"Use the flux criterion (average expected particle number) to determine when to terminate evaporation (1 or 0)."<<endl
	    <<"nbodyaverage:"<<endl
	    <<"1"<<endl
	    <<""<<endl
	    <<"Use kinematic cut-off to determine when to terminate evaporation (1 or 0)."<<endl
	    <<"kincut:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Bypass evaporation step and go directly from production to the remnant phase (1 or 0)."<<endl
	    <<"skip2remnant:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"--------------------------------------------"<<endl
	    <<"---------- Remnant model switches ----------"<<endl
	    <<"--------------------------------------------"<<endl
	    <<""<<endl
	    <<"Number of particles to which the remnant decays to."<<endl
	    <<"(Default is 2)"<<endl
	    <<"RMdecaynum:"<<endl
	    <<"2"<<endl
	    <<""<<endl
	    <<"Use uniform phase space for remnant decay (1) or use angular distributions associated to the remnant spin (0)."<<endl
	    <<"RMphasedecay:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Use the variable-multiplicity remnant model (1 or 0)."<<endl
	    <<"RMvarMultiplicity:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Allow the evaporation to continue until the maximum Hawking's temperature (defined in THmax) is reached (1 or 0)."<<endl
	    <<"ALso known as the boiling remnant model."<<endl
	    <<"RMboil:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Use a stable remnant model (the remnant will never fully decay (1 or 0)."<<endl
	    <<"RMstable:"<<endl
	    <<"0"<<endl
	    <<""<<endl
	    <<"Minimum mass required for the remnant boiling model in GeV (assumes RMboil=1)."<<endl
	    <<"RMminmass:"<<endl
	    <<"100"<<endl
	    <<""<<endl
	    <<""<<endl
	    <<""<<endl
	    <<"---END OF INPUT---"<<endl
	    <<endl;
}

void LH_block::print_init_tag(ofstream& output){
/******creates the standard <init> tag for the les houches file*******/
	output<<"<init>"<<endl
		<<"\t"<<IDbmup[0]<<"\t"<<IDbmup[1]<<"\t"<<Ebmup[0]<<"\t"<<Ebmup[1]<<"\t"<<PDFGup[0]<<"\t"<<PDFGup[1]<<"\t"<<PDFSup[0]<<"\t"<<PDFSup[1]<<"\t"<<IDwtup<<"\t"<<Nprup<<endl
		<<"\t"<<xsecup<<"\t"<<xerrup<<"\t"<<xmaxup<<"\t"<<Lprup<<endl
		<<"</init>"<<endl;
}
