#ifndef CHARYBDIS_CCINTERFACE_H
#define CHARYBDIS_CCINTERFACE_H
#include <string>
#include <iostream>
#include <ostream>
#include <fstream>
#include <cstdlib>
using namespace std;

class LH_block {
public:
  LH_block():IDwtup(3),Nprup(1),Lprup(1){PDFGup[0]=-1;PDFGup[1]=-1;}	//some variables are just are fixed and only initialized to be used in the output files
/*********public data***********/
  int IDbmup[2];		//particle ID's for the beams
  double Ebmup[2];		//beam energy
  string PDFname[2];	//PDF group (from LHAPDF)
  int PDFGup[2];		//les houches file pdf group code, at the moment LHAPDF seems to have a string name for this so it will be set to -1
  int PDFSup[2];		//PDF set   (from LHAPDF)
  int PDFnum[2];		//PDF member number (from LHAPDF)
  int IDwtup,Nprup,Lprup;	//fixed variables
  double xsecup,xerrup,xmaxup;	//cross-section: xsecup=average,xerrup=error,xmaxup=maximum
/*******class functions*********/  
  void print_init_tag(ofstream&);	//prints the <init></init> data in the file (see 
private:
};

class ccinterface {
 public:
  ccinterface(int, char**);   //CONSTRUCTOR
  friend class BHgenerator;   //let BH generator access class members (it needs acess to the input and output streams)
  friend class PDFs;		  //lets PDFs access class members (needs access to 
  void resetbufs();           //rests the cout,clog &cerr bufs if called
  void close_all_streams();
  int seed();
 private:
  /******************Original bufs******************/
  streambuf *coutbuf, *clogbuf, *cerrbuf;
  /*******************Streams Data******************/
  ifstream input;                         //input stream
  ofstream lhevent, eventhistory;         //output streams (Les houches event file & event history)
  ofstream newcout, newclog, newcerr;     //cout,clog & cerr destination if redirected
  bool rdcout,rdclog,rdcerr;              //true if streams are redirected - false if not
  /***********************Input data*******************/
  int argnumber;				//argument counter to check if no invalid ones were given
  LH_block LH_head;         //Les houches block
  string endinput;            //string that will be used to detect the end of input (initialized in the constructor)
  double minBHmass;           //minimum BH mass accepted during production (in GeV)
  double maxBHmass;           //maximum BH mass c during production (in GeV)
  int NumMaxEvents;           //number of events to generate
  int mc_samplesize;		  //monte carlo sample size
  int randseed;               //seed for the random number generator
  bool BH_in_LHE;             //defines if the black hole is appears as an intermediary particle on the Les Houches file or not
  int verbositycode;          //defines the amount of information to output to cout
  int TotalDim;               //number of dimensions to use
  double PlanckMass;          //planck mass to use (in GeV)
  int planckmassdef;          //switch that defines how planck's mass is calculated
  int enhanceXsec;          //switch that defines the use Yoshino-Rychkov impact cross-section calculation
  bool MJlostGraviton;        //switch that defines if there is mass loss threw gravity waves in the balding phase
  bool useminBHmass;          //switch that defines if there is a min BH mass at evaporation or if it goes down to planck scale
  bool cvbias;                //switch that defines the use of the constant angular velocity bias
  double isotroploss;         //loss of isotropy threw gravitational radiation
  bool GTscale;               //switch that defines the use of Giddings-Thomas scale for the PDF's
  bool nleptonicsv[4];        //switchs that define if TOTAL leptonic number[0], leptonic [1], muonic  [2] and tau [3] conservation is imposed
  bool BHspin;                //switch that defines if the black hole has angular momentum
  bool BHspinvar;             //switch that defines if the angular momentum may vary during the evaporation phase
  bool BHanis;                //switch that defines the use non-uniform angular functions during evaporation
  bool greybody;              //switch that defines the use of grey-body factors
  bool THvar;                 //switch that defines if the Hawking's temperature may vary during evaporation
  int alloweddecay;           //switch that defines which particles may be produced during evaporation
  int BHrecoil;               //switch that defines the BH recoil model
  double THmax;               //maximum Hawking's temperature allowed
  bool nbodyaverage;          //switch that defines if the average expected particle number should be used to terminate evaporation
  bool kincut;                //switch that defines if the kinematic cut-off should be used to terminate evaporation
  bool skip2remnant;          //switch used to skip the evaporation phase directly to the remnant
  int RMdecaynum;             //defines the number of particles the remnant decays to
  bool RMphasedecay;          //switch that defines if remnant decay uniform ("phase-wise") or depends on remnant spin
  bool RMvarMultiplicity;     //switch that defines the use of the variable-multiplicity remnant model or not
  bool RMboil;                //switch that defines the use of the boiling remnant model or not
  bool RMstable;              //switch that defines the use of the stable remnant model or not
  double RMminmass;           //remnant minimum mass for the boiling model
  /**************************arguments and streams functions****************/
  void checkhelp(int, char**);            //checks if the --help command was given in the console
  void model_file(int, char**);           //checks if the --model command was given in the console
  void print_model();                     //prints the model input.init file if the command was given
  void help();                            //displays the help instructions
  void open_input(int,char**,string);     //opens the input file
  void no_cc_msg();                       //calls help and prompts the user for an input file
  bool open_output(int,char**,ofstream&,string,string);       //opens the output files
  bool RedirectOstream(ostream&,ofstream&,int,char**,string); //redirects the ostreams
  void checkfile(ifstream&,string);       //checks if the file was opened correctly
  void checkfile(ofstream&,string);       //checks if the file was opened correctly
  void check_model_file();
  /*******************************Input functions***************************/
  void input_and_headers();           //calls checkinput() on every variable and creates the output headers
  void checkinput(int&,string);       //looks for a string with a variable name, exctracts the value in the next line
  void checkinput(double&,string);    //and prints the information on the output headers
  void checkinput(bool&,string);      //(overloaded for the different variable types)
  void checkinput(string&,string);      //
  void outputheaderinfo(ofstream&);   //creates an information header on the output file  
};

#endif
