#include "charybdis_event.h"

/********************static class data*******************/
CRandomMersenne BHparticle::temp(0);					//temporary random number generator used to initialize the reference;
CRandomMersenne& BHparticle::rand=BHparticle::temp;		//will not be used by the program after initialization
int BHparticle::enhanceXsec;
int BHparticle::TotalDim;
bool BHparticle::MJlostGraviton;
bool BHparticle::cvbias;
vector<double> BHparticle::MJlostdata_b;			//vectors created from data files with Yoshino-Rychkov
vector<double> BHparticle::MJlostdata_Mb;			//numerical calculated data




particle::particle(int id,double*P):IDup(id),Spinup(9){
/**********costructor for id + 3momentum*******/
    for (int i=0;i<3;++i)
        Pup[i]=P[i];
    Pup[4]=mass_from_id(IDup);												//get the mass from the particle ID
    Pup[3]=sqrt(pow(Pup[0],2)+pow(Pup[1],2)+pow(Pup[2],2)+pow(Pup[4],2));	//E=sqrt(px²+py²+pz²+m²)
}

particle::particle(int id,double*P,int spin):IDup(id),Spinup(spin){
/*******costructor for id+3momentum+spin*******/
    for (int i=0;i<3;++i)
        Pup[i]=P[i];
    Pup[4]=mass_from_id(IDup);												//get the mass from the particle ID
    Pup[3]=sqrt(pow(Pup[0],2)+pow(Pup[1],2)+pow(Pup[2],2)+pow(Pup[4],2));	//E=sqrt(px²+py²+pz²+m²)
}

particle::particle(int id,double Pz):IDup(id),Spinup(9){					//beam partiles are non-polarized
/*******costructor for id+3momentum+spin*******/
    Pup[0]=0.0;
    Pup[1]=0.0;
    Pup[2]=Pz;
    Pup[4]=mass_from_id(IDup);												//get the mass from the particle ID
    Pup[3]=sqrt(pow(Pup[0],2)+pow(Pup[1],2)+pow(Pup[2],2)+pow(Pup[4],2));	//E=sqrt(px²+py²+pz²+m²)
}

BHparticle::BHparticle(particle& incoming_part1,particle& incoming_part2){
/**************BH constructor for the 1st black hole**************/	
	for (int i=0;i<4;++i)
		Pup[i]=incoming_part1.Pup[i]+incoming_part2.Pup[i];					//4-momentum of the BH= 4-momentum part1+ 4_momentum part2
	Pup[4]=sqrt(pow(Pup[3],2)-pow(Pup[0],2)-pow(Pup[1],2)-pow(Pup[2],2));	//mass=sqrt(E²-Px²-Py²-Pz²)	
	double yosh_Bmax[7]={1.145,1.333,1.441,1.515,1.570,1.613,1.648};		//Bmax factors from yoshino-Rychkov
	double b_frac=sqrt(rand.Random());
	double Bmax=1;
	if (enhanceXsec==1)
		Bmax=yosh_Bmax[TotalDim-5];						//Yoshino-Rychkov
	else if(enhanceXsec==2)
		Bmax=customBHmax(TotalDim-5);								//user defined function	

	double b_impact=b_frac*Bmax;						//impact parameter (in units of rs)
	//para non enhance+ yoshi crashar, criar warning/funcao caso user defined
	double J_frac=1,M_frac=1;
	if (enhanceXsec==1){

		int index_bup;
		for(int i=0;i<MJlostdata_b.size() && b_impact>MJlostdata_b[i];++i){
			index_bup=i;
		}
		double Mlb=MJlostdata_Mb[index_bup-1]+(MJlostdata_Mb[index_bup]-MJlostdata_Mb[index_bup-1])/(MJlostdata_b[index_bup]-MJlostdata_b[index_bup-1])*(b_impact-MJlostdata_b[index_bup-1]);				//MLB = MLBLO + (MLBHI - MLBLO) * (B-BLO)/(BHI - BLO)
		double Omega,astar;			//<--------nothing for now (for CVbias)
		do {
			M_frac=(1-Mlb)*sqrt(rand.Random())+Mlb;			//MFRAC = CHRGEN(I)**(0.5)*(1-MLB) + MLB
			J_frac=sqrt(rand.Random());					    //CHRGEN(I+1)**(0.5)
		}while(YRmodel_mirr(M_frac,J_frac,b_impact,Omega,astar)<Mlb);

	}

	Pup[4]*=M_frac;
		//b_impact igual a fatores na rotina CHYRMJLOST
		//////////////////////////////// Not finished!!! inconsistent momentum
	

}

double BHparticle::YRmodel_mirr(double M_frac,double J_frac, double b_impact,double Omega, double astar){
/****************balding phase acoording to yoshino-rychkov's model (arxiv:hep-th/0503171)****************/	
	double OmegaDm3=nsphere_area_factor(TotalDim-2);
	double OmegaDm2=nsphere_area_factor(TotalDim-1);
	double RHpow_r0=4*OmegaDm3*M_frac/((TotalDim-2)*OmegaDm2);		//RHPOW = (4*SPHDM3*M)/((D-2)*SPHDM2)
	double k1 = RHpow_r0;
    double k2=pow((TotalDim-2)*J_frac*b_impact/(4*M_frac),2);     //K2 = (((D-2)*J)/(4*M))**2 * B**2
	double rk=1,rkdiff;
	for (int i=0;i<100;++i){
		double f,fprime;
		if (TotalDim>=6){
			f=pow(rk,TotalDim-3)+k2*pow(rk,TotalDim-5)-k1;
			fprime=(TotalDim-3)*pow(rk,TotalDim-4)+k2*(TotalDim-5)*pow(rk,TotalDim-6)-k1;
		}
		else{
			f=k1*pow(rk,5-TotalDim)-pow(rk,2)-k2;
			fprime=k1*(5-TotalDim)*pow(rk,4-TotalDim)-2*rk;
		}
		rkdiff=f/fprime;
		rk-=rkdiff;
	}
	rk=abs(rk);
	rkdiff=abs(rkdiff);
	if (rkdiff>=0.0001*rk)
		return -1000;		//if there isn't a high enough precision, returns an error value (-1000)
	else
		return (TotalDim-2)*OmegaDm2/(4*OmegaDm3)*pow(RHpow_r0*rk,(TotalDim-3.0)/(TotalDim-2.0));//MIRR = ((D-2)*SPHDM2)/(4*SPHDM3) &  * (RHPOW * RK)**((D-3.0)/(D-2.0))
			//returns the irreductible mass
}

event::event(){
/************costructor for the event*************/	
	part.reserve(100);				//keeps memory allocated for 100 particles (for efficiency)
	blackhole.reserve(100);			//keeps memory allocated for 100 black holes (for efficiency)
}

void event::reset(){
/************resets the event********************/	
	blackhole.clear();
	part.clear();
	
}

double particle::mass_from_id(int id){
/***returns the rest mass of a particle (in GeV) given it's id***/	
	switch (abs(id)){				//particle masses:
	case (1):return 0.32;			//	down quark
	case (2):return 0.32;			//	up quark
	case (3):return 0.5;			//	strange quark
	case (4):return 1.55;			//	charm quark
	case (5):return 4.95;			//	bottom quark
	case (6):return 172.9;			//	top quark
	case (11):return 5.11e-4;		//	electron
	case (12):return 0.0;			//	electron neutrino
	case (13):return 0.10566;		//	muon
	case (14):return 0.0;			//	mu neutrino
	case (15):return 1.7771;		//	tau particle
	case (16):return 0.0;			//	tau netrino
	case (21):return 0.0;			//	gluon
	case (23):return 91.188;		//	Z boson
	case (24):return 80.42;			//	W boson
	case (25):return 126.0;			//higgs	
	default:cout<<"Error: mass_from_id was given an unexpected id value. ->"<<abs(id)<<endl;
			exit(EXIT_FAILURE);
	}	
}
